<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

/**
 * Email callback
 *
 * @author Igor Ratsun
 */
class Rtgc_email extends \Abstract_rtgc_callback
{

    public function column($value = '', $data = array())
    {
        return '<a href="mailto:' . $value . '">' . $value . '</a>';
    }

    public function field($name = '', $value = NULL)
    {
        return '<input type="email" name="' . $name . '" value="' . $value . '" placeholder="email@email.com" />';
    }

}
