.img_box,.img_box .photo{
    width:100px;
    height:100px
}.img_box:hover label{
    display:block !important;
    color:white;
    position: relative;
    top: 45%;
    background-color: red;
    font-weight: bold
}